@@ -1,3 +1,5 @@
+import logging
+import os
 from abc import ABCMeta
 from abc import abstractmethod
 from typing import Any
@@ -12,16 +14,19 @@ from sklearn.model_selection import ParameterGrid
 from sklearn.model_selection import ParameterSampler
 
 
-class BaseSearch(metaclass=ABCMeta):
-    """RandomSearch for a given parameter distribution.
+logger = tf.get_logger()
 
-        Args:
-            model_fn (Callable): Function that builds and compiles a
-                tf.keras.Model or tf.keras.Sequential object.
-            verbose (int, optional): Whether to show information in terminal.
-                Defaults to 0.
-            kwargs (Any): Keyword arguments for the model_fn function.
-        """
+
+class BaseSearch(metaclass=ABCMeta):
+    """Abstract BaseSearch class for the grid or random search.
+
+    Args:
+        model_fn (Callable): Function that builds and compiles a
+            tf.keras.Model or tf.keras.Sequential object.
+        verbose (int): Whether to show information in terminal.
+            Defaults to 0.
+        kwargs (Any): Keyword arguments for the model_fn function.
+    """
     @abstractmethod
     def __init__(
         self,
@@ -58,6 +63,22 @@ class BaseSearch(metaclass=ABCMeta):
             kwargs (Any): Keyword arguments for the fit method of the
                 tf.keras.models.Model or tf.keras.models.Sequential model.
         """
+
+        tensorboard_callback = None
+        tensorboard_log_dir = ""
+
+        for param, value in kwargs.items():
+            if param == "callbacks":
+                for callback in value:
+                    if isinstance(callback, tf.keras.callbacks.TensorBoard):
+                        tensorboard_callback = callback
+
+        if tensorboard_callback:
+            tensorboard_log_dir = tensorboard_callback.log_dir
+
+        tf_log_level = logger.level
+        logger.setLevel(logging.ERROR)  # Issue 30: Ignore warnings for training
+
         for idx, grid_combination in enumerate(parameter_obj):
             if self.verbose:
                 print(f"Running Comb: {idx}")
@@ -66,6 +87,13 @@ class BaseSearch(metaclass=ABCMeta):
                 **self.model_fn_kwargs
             )
 
+            if tensorboard_callback:
+                if not os.path.exists(tensorboard_log_dir):
+                    os.mkdir(tensorboard_log_dir)
+                new_log_dir = os.path.join(tensorboard_log_dir, f'model_{idx}')
+                os.mkdir(new_log_dir)
+                tensorboard_callback.log_dir = new_log_dir
+
             model.fit(
                 train_dataset,
                 validation_data=val_dataset,
@@ -75,16 +103,28 @@ class BaseSearch(metaclass=ABCMeta):
             val_metric = model.evaluate(
                 val_dataset,
                 verbose=0
-            )[1]
+            )[-1]
             self.results_["val_scores"].append(val_metric)
             self.results_["params"].append(grid_combination)
 
+        logger.setLevel(tf_log_level)  # Issue 30
         best_run_idx = np.argmax(self.results_["val_scores"])
         self.results_["best_score"] = self.results_["val_scores"][best_run_idx]
         self.results_["best_params"] = self.results_["params"][best_run_idx]
 
     def summary(self) -> None:
         """Prints the summary of the search to the console.
+
+        Assuming the *RandomSearch* had n iterations or the
+        *GridSearch* had n combinations in total, the output
+        would have the following structure::
+            --------------------------------------------------
+            Best score: ``float`` using params: ``dict``
+            --------------------------------------------------
+            Idx: 0   - Score: ``float`` using params: ``dict``
+            ...
+            Idx: n-1 - Score: ``float`` using params: ``dict``
+            --------------------------------------------------
         """
         best_params_str = (f"Best score: {self.results_['best_score']} "
                            f"using params: {self.results_['best_params']}")
@@ -105,19 +145,27 @@ class GridSearch(BaseSearch):
         self,
         model_fn: Callable,
         param_grid: Mapping,
-        n_iter: int = 10,
         verbose: int = 0,
         **kwargs: Any
     ) -> None:
-        """RandomSearch for a given parameter distribution.
+        """GridSearch for a given parameter grid.
+
+        The grid search is evaluated by the either the validation loss value,
+        if no metrics are passed to the compile function, otherweise the
+        validation score of the last defined metric is used.
+        .. code-block:: python
+
+            model.compile(loss="mse", metrics=["mse", "mae"])
+
+        This would sort the grid search combinations based on the validation
+        mae score.
 
         Args:
             model_fn (Callable): Function that builds and compiles a
                 tf.keras.Model or tf.keras.Sequential object.
-            param_grid (Dict[str, Iterable]): Dict of str, iterable
+            param_grid (Mapping): Dict of str, iterable
                 hyperparameter, where the str is the parameter name of the.
-            n_iter (int, optional): Number of random models. Defaults to 10.
-            verbose (int, optional): Whether to show information in terminal.
+            verbose (int): Whether to show information in terminal.
                 Defaults to 0.
             kwargs (Any): Keyword arguments for the model_fn function.
         """
@@ -127,7 +175,6 @@ class GridSearch(BaseSearch):
             **kwargs
         )
         self.param_grid = ParameterGrid(param_grid)
-        self.n_iter = n_iter
 
     def fit(
         self,
@@ -164,13 +211,23 @@ class RandomSearch(BaseSearch):
     ) -> None:
         """RandomSearch for a given parameter distribution.
 
+        The random search is evaluated by the either the validation loss value,
+        if no metrics are passed to the compile function, otherweise the
+        validation score of the last defined metric is used.
+        .. code-block:: python
+
+            model.compile(loss="mse", metrics=["mse", "mae"])
+
+        This would sort the random search combinations based on the validation
+        mae score.
+
         Args:
             model_fn (Callable): Function that builds and compiles a
                 tf.keras.Model or tf.keras.Sequential object.
             param_distributions (Dict[str, Callable]): Dict of str, callable
                 pairs, where the str is the parameter name of the.
-            n_iter (int, optional): Number of random models. Defaults to 10.
-            verbose (int, optional): Whether to show information in terminal.
+            n_iter (int): Number of random models. Defaults to 10.
+            verbose (int): Whether to show information in terminal.
                 Defaults to 0.
             kwargs (Any): Keyword arguments for the model_fn function.
         """
